selectComboStrings <- function(type) {
    if (length(type) > 1) {
        if (length(type) > 3)
            out <- c("DropDown", "TypeIn", "Radio")
        else
            out <- c("Radio", "Typein", "DropDown")
    }
    else {
        if (type == "logical")
            out <- c("Radio", "TypeIn", "DropDown")
        else
            out <- c("TypeIn", "DropDown", "Radio")
    }

    out
}

extractArgs <- function(argTable) {
    ## Arg table unrolls in such a way that the last
    ## element of the children list is 0/0 and the first is X/Y
    ## so reverse it for easier calculations

    argTable <- rev(argTable)

    ## We currently have 6 columns, so can get the # of rows
    nRows <- (length(argTable) / 6) - 1

    outList <- vector(mode="list", length=nRows)

    for (i in 1:nRows) {
        root <- i * 6

        type <- gtkEntryGetText(argTable[[root+2]])
        def <- gtkEntryGetText(argTable[[root+3]])
        loc <- gtkEntryGetText(argTable[[root+4]])
        widget <- gtkEntryGetText(gtkContainerGetChildren(argTable[[root+5]])[[1]])
        req <- gtkToggleButtonGetActive(gtkContainerGetChildren(argTable[[root+6]])[[2]])

        outList[[i]] <- list(type=type, default=def, location=loc,
                             widget=widget, required=req)
    }
    outList
}


checkArgDefault <- function(type, default) {
    if ((type != "ANY")&&(length(type) > 0)) {
        if (length(type) > 1) {
            if (! default %in% type)
                paste("Default", default, "not valid for type",
                      paste(type, collapse=", "))
        }
        else {
            if (type == "character") {
                default <- gsub("^\"","", default)
                default <- gsub("\"$", "", default)
            }
            checkExpr <- paste("as.", type, "(\"", default, "\")", sep="")
            if ((nchar(default) == 0) ||
                (!is.na(eval(parse(text=checkExpr)))))
                character()
            else
                paste("Default", default, "not valid for type", type)
        }
    }
    else
        character()
}

checkArgWidget <- function(type, widget) {
    ## Will sanity check the widget selection against the type

    out <- character()

    if (length(type) > 0) {
        if ((length(type) > 1) || (type == "logical")) {
            if (! widget %in% c("Radio", "DropDown")) {
                out <- "Invalid widget type"
            }
        } else
        if (widget != "TypeIn")
            out <- "Invalid widget type"
    }
    out
}

checkErrorWindow <- function(msg) {
    errWin <- gtkWindow(show=FALSE)
    gtkWindowSetModal(errWin, TRUE)
    mbox <- gtkVBox()
    mbox$PackStart(gtkLabel("The following problems were found"))
    mbox$PackStart(gtkHSeparatorNew())
    for (i in 1:length(msg)) {
        if (length(msg[i]) > 0)
            mbox$PackStart(gtkLabel(msg[i]))
    }
    mbox$PackStart(gtkHSeparatorNew())

    okB <- gtkButton("OK")
    okB$AddCallback("clicked", function(obj) errWin$Destroy())
    mbox$PackStart(okB)
    errWin$Add(mbox)
    errWin$Show()
}

checkOKWindow <- function() {
    labelOKDialog("No problems detected")
}

getCurWF <- function(wFun, win) {
    curArgs <- gtkContainerGetChildren(gtkContainerGetChildren(gtkContainerGetChildren(win)[[1]])[[3]])

    wfunArgList <- funArgList(wFun)
    argList <- extractArgs(curArgs)

    for (i in 1:length(argList)) {
        curArgList <- argList[[i]]
        wfunArgList[[i]]@argDefault <- curArgList$default
        if (nchar(curArgList$type) > 0)
            argType <- strsplit(curArgList$type, ", ")[[1]]
        else
            argType <- "ANY"
        wfunArgList[[i]]@argType <- argType
        wfunArgList[[i]]@argLocation <- curArgList$location
        wfunArgList[[i]]@argWidgetType <- curArgList$widget
        wfunArgList[[i]]@argRequired <- curArgList$required
    }
    wFun@funArgList <- wfunArgList
    wFun
}

resetWF <- function(oWF, win) {
    argTable <- gtkContainerGetChildren(gtkContainerGetChildren(gtkContainerGetChildren(win)[[1]])[[3]])

    wfunArgList <- funArgList(oWF)

    argTable <- rev(argTable)

    ## We currently have 6 columns, so can get the # of rows
    nRows <- (length(argTable) / 6) - 1

    outList <- vector(mode="list", length=nRows)

    for (i in 1:nRows) {
        root <- i * 6

        gtkEntrySetText(argTable[[root+2]],
                        paste(argType(wfunArgList[[i]]), collapse=", "))
        gtkEntrySetText(argTable[[root+3]], argDefault(wfunArgList[[i]]))
        gtkEntrySetText(argTable[[root+4]], argLocation(wfunArgList[[i]]))
        gtkEntrySetText(gtkContainerGetChildren(argTable[[root+5]])[[1]],
                        argWidgetType(wfunArgList[[i]]))
        gtkToggleButtonSetActive(gtkContainerGetChildren(argTable[[root+6]])[[2]], argRequired(wfunArgList[[i]]))
    }
}


createWF <- function(funName) {
    curSaveFile <- file.path(getwd(), paste(funName, ".xml", sep=""))

    if (file.exists(curSaveFile))
        origWF <- wFun <- readWIxml(curSaveFile)
    else
        origWF <- wFun <- fun2wFun(funName)

    checkArgs <- function(obj) {
        argNames <- sapply(funArgList(wFun), argName)

        curArgs <- gtkContainerGetChildren(gtkContainerGetChildren(gtkContainerGetChildren(win)[[1]])[[3]])
        argList <- extractArgs(curArgs)


        ## Now have a list of GtkHBox entities, each one is one arg
        ## FIXME: run this thru a sapply later
        outList <- vector(mode="list", length=length(curArgs))
        for (i in 1:length(argList)) {

            argOut <- character()

            argType <- argList[[i]]$type
            if (nchar(argType) > 0)
                argType <- strsplit(argType, ", ")[[1]]
            else
                argType <- "ANY"

            if (length(argList[[i]]$default) > 0) {
                defOut <- checkArgDefault(argType, argList[[i]]$default)
                if (length(defOut) > 0)
                    argOut <- c(argOut, paste(argNames[i],": ", defOut, sep=""))
            }

            widOut <- checkArgWidget(argType, argList[[i]]$widget)
            if (length(widOut) > 0)
                argOut <- c(argOut, paste(argNames[i],": ", widOut, sep=""))

            outList[[i]] <- argOut
        }
        errOut <- unlist(outList)
        if (any(length(errOut) > 0)) {
            checkErrorWindow(errOut)
            return(FALSE)
        }
        else {
            checkOKWindow()
            return(TRUE)
        }
    }
    saveArgs <- function(obj) {
        wFun <- getCurWF(wFun, win)
        ## FIXME: Make sure we're not overwriting anything
        print(curSaveFile)
        z <- try(writeWIxml(wFun, curSaveFile))

        if (!inherits(z, "try-error"))
            labelOKDialog("Function arguments successfully saved")
        else
            labelOKDialog("Error in saving function arguments")
    }

    win <- gtkWindow(show=FALSE)
    gtkWindowSetTitle(win, funName(wFun))

    mainBox <- gtkVBox()
    titleTab <- gtkTable(1, 8, TRUE)
    titleTab$Attach(gtkLabel(paste("Function:", funName(wFun))), 0, 1,
                    0, 1)

    fileNameLab <- gtkLabel(paste("Filename: ", curSaveFile))
    titleTab$Attach(fileNameLab, 2, 6, 0, 1)

    saveAsBut <- gtkButton("Save As")
    saveAsBut$AddCallback("clicked", function(obj) {
        curSaveFile <<- saveAs(curSaveFile)
        gtkLabelSetText(fileNameLab, paste("Filename:", curSaveFile))
    })

    titleTab$Attach(saveAsBut, 6, 7, 0, 1)


    manBut <- gtkButton("Man Page")
    manBut$AddCallback("clicked", function(obj) dispMan(funName(wFun)))

    titleTab$Attach(manBut, 7, 8, 0, 1)

    mainBox$PackStart(titleTab)
    mainBox$PackStart(gtkHSeparatorNew())

    argList <- funArgList(wFun)
    tbl <- gtkTable(length(argList)+1, 6, FALSE)
    tbl$SetRowSpacings(10)
    tbl$SetColSpacings(10)

    tbl$Attach(gtkLabel("Argument"), 0, 1, 0, 1)
    tbl$Attach(gtkLabel("Type"), 1, 2, 0, 1)
    tbl$Attach(gtkLabel("Default"), 2, 3, 0 ,1)
    tbl$Attach(gtkLabel("Location"), 3, 4, 0, 1)
    tbl$Attach(gtkLabel("Widget Type"), 4, 5, 0, 1)
    tbl$Attach(gtkLabel("Required  "), 5, 6, 0, 1)

    for (i in 1:length(argList)) {
        ## The actual argument name
        tbl$Attach(gtkLabel(paste(" ", argName(argList[[i]]))),
                   0, 1, i, i+1)

        typeEntry <- gtkEntry()
        gtkEntrySetText(typeEntry, paste(argType(argList[[i]]), collapse=", "))
        tbl$Attach(typeEntry, 1, 2, i, i+1)

        defEntry <- gtkEntry()
        gtkEntrySetText(defEntry, argDefault(argList[[i]]))
        tbl$Attach(defEntry, 2, 3, i, i+1)

        paneEntry <- gtkEntry()
        if (length(argLocation(argList[[i]])) == 0)
            pane <- "main"
        else
            pane <- argLocation(argList[[i]])
        gtkEntrySetText(paneEntry, as.character(pane))
        tbl$Attach(paneEntry, 3, 4, i, i+1)

        widgetCombo <- gtkCombo()
        widgetCombo$SetPopdownStrings(selectComboStrings(argType(argList[[i]])))
        tbl$Attach(widgetCombo, 4, 5, i, i+1)

        reqBox <- gtkHBox()
        reqBox$PackStart(gtkLabel("  "))
        reqCheckBox <- gtkCheckButton()
        if (argRequired(argList[[i]]))
            gtkToggleButtonSetActive(reqCheckBox, TRUE)
        reqBox$PackStart(reqCheckBox)
        tbl$Attach(reqBox, 5, 6, i, i+1)
    }

    mainBox$PackStart(tbl)

    mainBox$PackStart(gtkHSeparatorNew())

    ## Now need the buttons on the bottom
    butBox <- gtkHBox()

    canB <- gtkButton("Close")
    canB$AddCallback("clicked", function(obj) win$Destroy())
    butBox$PackStart(canB)

    resB <- gtkButton("Reset")
    resB$AddCallback("clicked", function(obj) resetWF(origWF, win))
    butBox$PackStart(resB)

    chkB <- gtkButton("Check")
    chkB$AddCallback("clicked", function(obj) checkArgs(obj))
    butBox$PackStart(chkB)

    preB <- gtkButton("Preview")
    preB$AddCallback("clicked", function(obj)
                     renderW(getCurWF(wFun, win), TRUE))
    butBox$PackStart(preB)

    savB <- gtkButton("Save")
    savB$AddCallback("clicked", function(obj) saveArgs(obj))
    butBox$PackStart(savB)

    mainBox$PackEnd(butBox)

    win$Add(mainBox)
    win$Show()
}

labelOKDialog <- function(label) {
        labWin <- gtkDialogNew(show=FALSE)
        gtkWindowSetTitle(labWin, "")

        gtkWindowSetModal(labWin, TRUE)
        labWin[["Vbox"]]$PackStart(gtkLabel(label))

        okB <- gtkButton("OK")
        okB$AddCallback("clicked", function(obj) labWin$Destroy())
        labWin[["ActionArea"]]$PackStart(okB)

        labWin$Show()
}

dispMan <- function(funName) {
    gtkTextSetText <-
        function(w, contents="", append = FALSE)
        {
            if(append == FALSE) {
                gtkTextClearText(w, )
            }

            .Call("R_gtk_setTextContents", w, as.character(contents),
                  as.logical(append), PACKAGE="RGtk")
        }
    man <- system.file("help", funName,
                       package=getFunPackage(funName))
    w <- gtkWindow(show = FALSE)
    w$SetUsize(500, 500)
    box <- gtkVBox()
    txt <- gtkText()
    gtkAdd(box, txt)
    gtkAdd(w, box)
    gtkShow(txt, box, w)
    x1=scan(man, what="", sep="\n")
    x2= paste(x1, collapse="\n")
    gtkTextSetText(txt, x2)
}

saveAs <- function(fileName) {
    fsEnv <- new.env()
    assign("finished", FALSE, fsEnv)

    fileSelect <- gtkFileSelection("Save File As ...")
    gtkFileSelectionHideFileopButtons(fileSelect)
    gtkFileSelectionSetFilename(fileSelect, fileName)
    gtkWindowSetModal(fileSelect, TRUE)

    fsOK <- fileSelect[["OkButton"]]
    fsOK$AddCallback("clicked", function(x) {
        out <- gtkFileSelectionGetFilename(fileSelect)
        fileSelect$Destroy()
        assign("out", out, fsEnv)
        assign("finished", TRUE, fsEnv)
    })

    fsCan <- fileSelect[["CancelButton"]]
    fsCan$AddCallback("clicked", function(x) {
        fileSelect$Destroy()
        assign("out", fileName, fsEnv)
        assign("finished", TRUE, fsEnv)
    })

    while (get("finished", fsEnv) == FALSE)
        Sys.sleep(0.5)
    out <- get("out", fsEnv)
    if (file.info(out)$isdir)
        file.path(out, basename(fileName))
    else
        out
}

