controlEnv <- new.env()

setClass("SimpleW", representation(defaultValue="character"),
         prototype(defaultValue=""))

if (is.null(getGeneric("defaultValue")))
    setGeneric("defaultValue", function(object)
               standardGeneric("defaultValue"))

setClass("RadButtonW", representation(nItems="numeric",
                                      itemNames="character"),
         contains="SimpleW", prototype(default=""))

if (is.null(getGeneric("nItems")))
    setGeneric("nItems", function(object)
               standardGeneric("nItems"))
setMethod("nItems", "RadButtonW", function(object)
          object@nItems)

if (is.null(getGeneric("itemNames")))
    setGeneric("itemNames", function(object)
               standardGeneric("itemNames"))
setMethod("itemNames", "RadButtonW", function(object)
          object@itemNames)

setMethod("defaultValue", "RadButtonW", function(object)
          object@defaultValue)

setClass("DropDownW", representation(nItems="numeric",
                                     itemNames="character"),
         contains="SimpleW")

if (is.null(getGeneric("nItems")))
    setGeneric("nItems", function(object)
               standardGeneric("nItems"))
setMethod("nItems", "DropDownW", function(object)
          object@nItems)

if (is.null(getGeneric("itemNames")))
    setGeneric("itemNames", function(object)
               standardGeneric("itemNames"))
setMethod("itemNames", "DropDownW", function(object)
          object@itemNames)

setMethod("defaultValue", "DropDownW", function(object)
          object@defaultValue)

setClass("TypeInW", representation(returnType="character"),
         contains="SimpleW")

setMethod("defaultValue", "TypeInW", function(object)
          object@defaultValue)

if (is.null(getGeneric("returnType")))
    setGeneric("returnType", function(object)
               standardGeneric("returnType"))
setMethod("returnType", "TypeInW", function(object)
          object@returnType)


buildRadButW <- function(funArg) {
    ## FIXME: Need to handle all types of arguments

    type <- argType(funArg)
    if (length(type) > 1) {
        nItems <- length(type)
        itemNames <- type
    }
    else {
        if (type == "logical") {
            itemNames <- c("TRUE", "FALSE")
            nItems <- 2
        }
        else
            stop("Don't know how to construct radio buttons for type ",
                 type)
    }

    new("RadButtonW", nItems=nItems, itemNames=itemNames,
        defaultValue=argDefault(funArg))
}

buildDropDownW <- function(funArg) {
    type <- argType(funArg)

    new("DropDownW", nItems=length(type),
        itemNames=type, defaultValue=argDefault(funArg))
}

buildTypeInW <- function(funArg) {
    new("TypeInW", defaultValue=argDefault(funArg),
        returnType=argType(funArg))
}


buildW <- function(funArg) {
    if (! is(funArg, "funArg"))
        stop("buildW requires objects of class 'funArg'")

    type <- argType(funArg)

    switch(argWidgetType(funArg),
           "TypeIn" = buildTypeInW(funArg),
           "Radio" = buildRadButW(funArg),
           "DropDown" = buildDropDownW(funArg),
           stop("Invalid widget type for argument: ", argWidgetType(funArg)))
}

retrieveTIW <- function(TIWcont) {
    gtkEntryGetText(gtkContainerGetChildren(TIWcont)[[1]])
}

retrieveDDW <- function(DDWcont) {
    gtkEntryGetText(gtkContainerGetChildren(gtkContainerGetChildren(DDWcont)[[1]])[[1]])
}

retrieveRBW <- function(RBWcont) {
    z <- gtkContainerGetChildren(RBWcont)
    if (length(z) >= 2) {
        for (i in 1:length(z)) {
            val <- gtkToggleButtonGetActive(z[[i]])
            if (val == TRUE)
                break()
        }
        return(i)
    }
    else
        stop("Invalid RadioButton widget, no buttons")
}

renderRBW <- function(curName, funW) {
    if (! is(funW, "RadButtonW"))
        stop("renderRBW requires objects of class RadButtonW")

    names <- itemNames(funW)
    def <- defaultValue(funW)

    box <- gtkHBox(TRUE, 1)
    b <- gtkRadioButton(NULL, names[1])
    box$PackStart(b)
    for (i in 2:length(names)) {
        b1 <- gtkRadioButton(b, names[i])

        if ((length(def) > 0)&&(names[i] == def))
            gtkToggleButtonSetActive(b1, "TRUE")

        box$PackStart(b1)
    }

    box
}

renderDDW <- function(curName, funW) {
    if (! is(funW, "DropDownW"))
        stop("renderDDW requires objects of class DropDownW")

    ## Try to get the default value first
    pos <- match(defaultValue(funW), itemNames(funW))
    if (! is.na(pos)) {
        vals <- itemNames(funW)[-pos]
        vals <- c(defaultValue(funW), vals)
    }
    else
        vals <- itemNames(funW)

    out <- gtkHBox(FALSE, 3)

    combo <- gtkCombo()
    combo$SetPopdownStrings(vals)

    out$PackStart(combo)
    out
}

renderTIW <- function(curName, funW) {
    if (! is(funW, "TypeInW"))
        stop("renderTIW requires objects of class TypeInW")

    out <- gtkHBox(FALSE, 3)
    entry <- gtkEntry()
    gtkEntrySetText(entry, defaultValue(funW))
    out$PackStart(entry)
    out
}

renderW <- function(wFun, modal=FALSE) {

    assign("finished", FALSE, controlEnv)

    evalClicked <- function(x) {
        wBook <- gtkContainerGetChildren(gtkContainerGetChildren(gtkContainerGetChildren(win)[[1]])[[3]])
        wBook <- lapply(wBook, function(x) {rev(gtkContainerGetChildren(x))})

        curWid <- rep(2, length(panes))

        err <- character()

        for (i in 1:length(widgetList)) {
            curPane <- match(locationMap[i], names(panes))

            curTab <- wBook[[curPane]]
            if (is(widgetList[[i]], "RadButtonW")) {
                pos <- retrieveRBW(curTab[[curWid[curPane]]])
                out[[i]] <- itemNames(widgetList[[i]])[pos]
            }
            else if (is(widgetList[[i]], "DropDownW")) {
                out[[i]] <- retrieveDDW(curTab[[curWid[curPane]]])
            }
            else if (is(widgetList[[i]], "TypeInW")) {
                out[[i]] <- retrieveTIW(curTab[[curWid[curPane]]])
            }

            if ((argReqs[i] == TRUE)&&(out[[i]] == "")) {
                err <- c(err,
                         paste(argNames[i], "is a required argument."))
            }

            curWid[curPane] <- curWid[curPane] + 2
        }

        if (length(err) > 0)
            out <- err

        out
    }

    funName <- funName(wFun)
    argNames <- sapply(funArgList(wFun), argName)
    argReqs <- sapply(funArgList(wFun), argRequired)

    locationMap <- sapply(funArgList(wFun), argLocation)

    panes <- split(argNames, locationMap)

    mainPane <- match("main", names(panes))
    if (!is.na(mainPane))
        panes <- c(panes[mainPane], panes[-mainPane])

    required <- sapply(funArgList(wFun), argRequired)
    required <- argNames[which(required)]

    widgetList <- sapply(funArgList(wFun), buildW)

    out <- vector(mode="list", length=length(argNames))
    names(out) <- argNames

    boxList <- mapply(function(x, y) {
        if (is(x, "RadButtonW"))
            renderRBW(y, x)
        else if (is(x, "DropDownW"))
            renderDDW(y, x)
        else if (is(x, "TypeInW"))
            renderTIW(y, x)
    }, widgetList, argNames, SIMPLIFY=FALSE)

    win <- gtkWindow(show=FALSE)
    gtkWindowSetModal(win, modal)
    ## need a delete event callback in case the user closes
    ## the window without using the Quit menu item
    gtkAddCallback(win,"delete_event",
                   function(obj,ev) {
                       assign("output", FALSE, controlEnv)
                       assign("finished", TRUE, controlEnv)
                       win$Destroy()
                   })


    book <- gtkNotebook()
    book$SetTabPos("top")

    for (i in 1:length(panes)) {
        tab <- gtkTable(length(panes[[i]]), 2, FALSE)
        tab$SetRowSpacings(3)
        tab$SetColSpacings(5)
        ## FIXME: this can surely be done better
        cur <- 1
        for (j in seq(along=boxList)) {
            if (argNames[j] %in% panes[[i]]) {
                if (argReqs[j] == TRUE)
                    outLab <- paste("*", argNames[j], "*", sep="")
                else
                    outLab <- argNames[j]

                tab$Attach(gtkLabel(outLab), 0, 1, cur-1, cur)
                tab$Attach(boxList[[j]], 1, 2, cur-1, cur)
                cur <- cur+1
            }
        }
        book$AppendPage(tab, gtkLabel(names(panes)[i]))
    }

    mainBox <- gtkVBox()

    titleTab <- gtkTable(1, 2, TRUE)
    titleTab$Attach(gtkLabel(paste("Function:", funName(wFun))), 0, 1,
                    0, 1)
    manBut <- gtkButton("Man Page")
    manBut$AddCallback("clicked", function(obj) dispMan(funName(wFun)))

    titleTab$Attach(manBut, 1, 2, 0, 1)

    mainBox$PackStart(titleTab)

    mainBox$PackStart(gtkHSeparatorNew())

    mainBox$PackStart(book)

    mainBox$PackStart(gtkHSeparatorNew())

    finished <- FALSE

    b <- gtkButton("Evaluate")
    b$AddCallback("clicked", function(obj) {
        if (is.list(out)) {
            out <- evalClicked(obj)
            win$Destroy()
            assign("output", out, controlEnv)
            assign("finished", TRUE, controlEnv)
        }
        else {
            ## Errors
            labelOKDialog(paste(out, collapse="\n"))
        }
    })

    mainBox$PackEnd(b)

    win$Add(mainBox)
    win$Show()

    ## Need to poll to simulate a modal window
    while (get("finished", controlEnv) == FALSE)
        Sys.sleep(0.5)

    get("output", controlEnv)
}


