setClass("funArg", representation(argName="character",
                                  argDefault="character",
                                  argType="character",
                                  argLocation="character",
                                  argWidgetType="character",
                                  argRequired="logical"),
         prototype(argDefault=""))

if (is.null(getGeneric("argName")))
    setGeneric("argName", function(object)
               standardGeneric("argName"))

setMethod("argName", "funArg", function(object)
          object@argName)

if (is.null(getGeneric("argDefault")))
    setGeneric("argDefault", function(object)
               standardGeneric("argDefault"))

setMethod("argDefault", "funArg", function(object)
          object@argDefault)

if (is.null(getGeneric("argType")))
    setGeneric("argType", function(object)
               standardGeneric("argType"))

setMethod("argType", "funArg", function(object)
          object@argType)

if (is.null(getGeneric("argLocation")))
    setGeneric("argLocation", function(object)
               standardGeneric("argLocation"))

setMethod("argLocation", "funArg", function(object)
          object@argLocation)

if (is.null(getGeneric("argRequired")))
    setGeneric("argRequired", function(object)
               standardGeneric("argRequired"))
setMethod("argRequired", "funArg", function(object)
          object@argRequired)


if (is.null(getGeneric("argWidgetType")))
    setGeneric("argWidgetType", function(object)
               standardGeneric("argWidgetType"))
setMethod("argWidgetType", "funArg", function(object)
          object@argWidgetType)

setClass("wFun", representation(funName="character",
                               funArgList="list"))

if (is.null(getGeneric("funName")))
    setGeneric("funName", function(object)
               standardGeneric("funName"))
setMethod("funName", "wFun", function(object)
          object@funName)

if (is.null(getGeneric("funArgList")))
    setGeneric("funArgList", function(object)
               standardGeneric("funArgList"))
setMethod("funArgList", "wFun", function(object)
          object@funArgList)



fun2wFun <- function(funName) {

    orig <- getOption("show.error.messages")
    on.exit(options(show.error.messages=orig), add=TRUE)
    options(show.error.messages=FALSE)
    args <- try(formals(funName))
    options(show.error.messages=orig)

    if (inherits(args, "try-error"))
        stop("Function ", funName, " is not available.  Make sure ",
             "you have loaded its package first.")

    argNames <-  names(args)

    argList <- mapply(function(x, y) {
        type <- class(x)
        if (type == "name")
            type <- ""

        if (type == "call") {
            type <- eval(x)
            x <- type
        }

        if (length(type) > 1) {
            type <- paste("\"", type, "\"", sep="")

            if (length(type) <= 3)
                widget <- "Radio"
            else
                widget <- "DropDown"
        }
        else {
            if (type == "logical")
                widget <- "Radio"
            else
                widget <- "TypeIn"
        }

        if (all(is.character(x)))
            argDefault <- paste("\"", x[1], "\"", sep="", collapse=", ")
        else
            argDefault <- as.character(x)[1]

        new("funArg", argName=y, argDefault=argDefault,
            argType=type, argLocation="main", argWidgetType=widget,
            argRequired=FALSE)
    }, args, argNames)

    new("wFun", funName=funName, funArgList=argList)
}

getFunPackage <- function(funName) {
    pkg <- find(funName, mode="function")
    if (length(pkg) == 0)
        stop("Function ", funName, " does not seem to currently exist.")
    gsub("package:", "", pkg)
}

modWfun <- function(wFun, argOverides=list()) {
    ## Used to modify the wFun from what comes out of the
    ## XML file.
    argNames <- sapply(funArgList(wFun), argName)

    aoNames <- names(argOverides)
    ## FIXME: make this not a loop
    for (i in seq(along=argOverides)) {
        pos <- match(aoNames[i], argNames)
        if (is.na(pos))
            next
        cur <- argOverides[[i]]
        if (!is.null(cur$argDefault))
            wFun@funArgList[[pos]]@argDefault <- cur$argDefault
        if (!is.null(cur$argType))
            wFun@funArgList[[pos]]@argType <- cur$argType
        if (!is.null(cur$argLocation))
            wFun@funArgList[[pos]]@argLocation <- cur$argLocation
        if (!is.null(cur$argWidgetType))
            wFun@funArgList[[pos]]@argWidgetType <- cur$argWidgetType
        if (!is.null(cur$argRequired))
            wFun@funArgList[[pos]]@argRequired <- cur$argRequired

    }

    wFun
}

widgetInvoke <- function(funName, argOverides=list(), argsOnly=FALSE) {
    ## read in XML file to get wFun
    pkg <- getFunPackage(funName)

    xmlFile <- system.file("wFun", paste(funName, ".xml", sep=""),
                           package=pkg)
    if (xmlFile == "")
        stop("Function ", funName, " does not have the ",
             "proper XML file required for 'widgetInvoke'")
    else
        wFun <- readWIxml(xmlFile)

    ## FIXME: Modify the wFun w/ specified defaults
    wFun <- modWfun(wFun, argOverides)

    ## Get the values for the arguments
    vals <- renderW(wFun)
    if ((is.logical(vals))&&(vals==FALSE))
        return(invisible(NULL))
    valNames <- names(vals)

    argTypes <- sapply(funArgList(wFun), argType)

    ## Get rid of vals that have "" as the value, as those
    ## need to be 'missing' when we call the function
    badVals <- unlist(lapply(vals, function(x) x == ""))
    badVals <- which(badVals)
    if (length(badVals) > 0) {
        vals <- vals[- badVals]
        argTypes <- argTypes[- badVals]
        valNames <- valNames[- badVals]
    }

    ## Need to massage 'vals'.  If quoted string, remove outer quotes
    ## Otherwise, if it exist()s, get val, otherwise do as.<type> on
    ## it.
    vals <- mapply(function(x, y) {
        ## split on commas for vectors - need to remove any whitespace
        ## before and after the commas as strsplit doesn't seem to
        ## work well with [[:space:]]*,[[:space:]]*
        x <- gsub("[[:space:]]*,[[:space:]]*", ",", x)
        x <- strsplit(x, ",")[[1]]
        ## Now remove any empty entries caused by errant commas
        emptyVals <- which(x == "")
        if (length(emptyVals) > 0)
        x <- x[-emptyVals]

        if (length(grep("^\".*\"$", x)) > 0) {
            x <- gsub("^\"","", x)
            gsub("\"$", "", x)
        }
        else {
            if (exists(x))
                get(x)
            else {
                if (length(y) == 1) {
                    ## If the type is "ANY", and the input wasn't
                    ## detected as a string or a variable, assume it
                    ## is numeric as it is the most likely type at
                    ## this point of time in the function
                    if (y == "ANY")
                        y <- "numeric"

                    asExpr <- paste("as.", y, "(x)", sep="")
                    eval(parse(text=asExpr))
                }
                else
                    x
            }
        }
    } , vals, argTypes, SIMPLIFY=FALSE)


    names(vals) <- valNames

    if (argsOnly)
        return(vals)


    ## run the function w/ passed in args
    do.call(funName(wFun), vals)
}

