writeWIxml <- function(wFun, file=paste(funName(wFun),"xml",sep=".")) {

    out <- "<?xml version=\"1.0\"?>\n"
    out <- paste(out,
                 "<wFun xmlns:bt=\"http://www.bioconductor.org/WINVOKE\">")
    out <- paste(out, "\n\t<funName>", funName(wFun),
                 "</funName>\n\t<funArgList>\n", sep="")

    ## Now add each argument
    argOut <- sapply(funArgList(wFun), function(x) {

        type <- argType(x)
        if (length(type) > 1)
            type <- paste(type, collapse=",")

        paste("\t\t<funArg>\n\t\t\t<argName>",
              argName(x), "</argName>\n\t\t\t<argDefault>",
              argDefault(x), "</argDefault>\n\t\t\t",
              "<argType>", type, "</argType>\n\t\t\t",
              "<argLocation>", argLocation(x),
              "</argLocation>\n\t\t\t<argWidgetType>",
              argWidgetType(x),"</argWidgetType>\n\t\t\t",
              "<argRequired>", argRequired(x),
              "</argRequired>\n\t\t</funArg>", sep="")
    })

    out <- paste(out, paste(argOut, collapse="\n"),
                 "\n\t</funArgList>\n</wFun>", sep="")

    cat(out, file=file)
}

readWIxml <- function(file) {
    require("XML") || stop("This function requires the XML package")

    if (file.exists(file)) {
        xml <- xmlTreeParse(file)$doc$children$wFun

        funName <- as.character(xmlChildren(xml[["funName"]])$text)[6]

        args <- xml[["funArgList"]]
        argList <- vector(mode="list", length=length(args))
        for (i in seq(along=args)) {
            curArg <- args[[i]]
            argName <- as.character(xmlChildren(curArg[["argName"]])$text)[6]
            argDefault <- as.character(xmlChildren(curArg[["argDefault"]])$text)[6]
            if (is.na(argDefault))
                argDefault <- ""

            argType <- as.character(xmlChildren(curArg[["argType"]])$text)[6]
            if (is.na(argType))
                argType <- "ANY"
            else
                argType <- strsplit(argType, ",")[[1]]


            argLocation <- as.character(xmlChildren(curArg[["argLocation"]])$text)[6]
            if (is.na(argLocation))
                argLocation <- numeric()

            argWidgetType <- as.character(xmlChildren(curArg[["argWidgetType"]])$text)[6]
            if (is.na(argWidgetType))
                argWidgetType <- character()

            argRequired <- as.logical(as.character(xmlChildren(curArg[["argRequired"]])$text)[6])
            if (is.na(argRequired))
                argRequired <- FALSE

            argList[[i]] <- new("funArg", argName=argName,
                                argDefault=argDefault,
                                argType=argType,
                                argLocation=argLocation,
                                argWidgetType=argWidgetType,
                                argRequired=argRequired)
        }

        new("wFun", funName=funName, funArgList=argList)
    }
    else
        stop("File ", file, " does not exist.")
}
